import pandas as pd
import datetime as dt
from datetime import timedelta


def _dates_between_dates(start, end):
    return list(pd.date_range(start + timedelta(1),
                              end - timedelta(1),
                              freq='d'))


def _is_overlapping(x1, x2, y1, y2):
    return not (x1 > y2 or y1 > x2)


def _is_overlapping_bis(x1, x2, y1, y2):
    return not (max(x1, y1) <= min(x2, y2))


def overlap(x1, x2, y1=None, y2=None):
    if y1 is None and y2 is None:
        return x2 - x1

    if y1 is None:
        y1 = Time.min

    if x1 == x2 or y1 == y2:
        return timedelta(0)

    if y2 is None:
        y2 = Time.max

    if not _is_overlapping(x1, x2, y1, y2):
        raise ValueError("Intervals does not overlaps")

    return min(x2, y2) - max(x1, y1)


class Time(dt.time):
    def seconds(self):
        return self.hour * 3600 + self.minute * 60 + self.second

    @staticmethod
    def from_seconds(seconds):
        secs = seconds % 60
        mins = int(seconds / 60) % 60
        hours = int(seconds / 3600) % 24
        return Time(hours, mins, secs)

    @staticmethod
    def from_time(dtime):
        return Time(dtime.hour, dtime.minute, dtime.second)

    @staticmethod
    def from_dt(dtime):
        return Time.from_time(dtime.time())

    def __sub__(self, other):
        return timedelta(seconds=self.seconds() - other.seconds())

    def __rsub__(self, other):
        return Time.from_time(other) - self


class WorkingTime:
    def __init__(self, weekends=None, holidays=None):
        # TODO check the variables
        self.weekends = weekends
        self.holidays = holidays

    def working_time(self, start, end, work_hours=None):
        if work_hours is None:
            work_hours = (Time(8), Time(16))

        work_hours = (Time.from_time(work_hours[0]),
                      Time.from_time(work_hours[1]))

        # If start > end, workingtime will be negative
        if start > end:
            return -1 * self.working_time(end, start, work_hours)

        # If the day changes between your workhours, its easier to calculate
        # the time your note working and substract
        if work_hours[0] > work_hours[1]:
            return ((end - start) -
                    self.working_time(start, end,
                                      tuple(reversed(work_hours))))

        # If the start time and the end time are the same we
        # return the workhours * days between. But it can't be done
        # like this because start date or end date could be weekend or holiday

        if start.date() == end.date():
            # TODO implementar esta  cosica
            return overlap(Time.from_dt(start), Time.from_dt(end), *work_hours)

        middle_time = (len(_dates_between_dates(start, end)) *
                       (work_hours[1] - work_hours[0]))

        return (overlap(*work_hours, Time.from_dt(start)) +
                middle_time +
                overlap(*work_hours, None, Time.from_dt(end)))



"""

# TODO main description

"""
import errno as _errno

from .workingtime import WorkingTime, Time

import os as _os

try:
    with open(_os.path.join(_os.path.dirname(__file__),
                            '..', 'VERSION'), 'r') as version_file:
        __version__ = version_file.read().strip()
except IOError as e:
    if e.errno != _errno.ENOENT:
        raise

    __version__ = "0.0"



# encoding: utf-8

"""
# TODO description
"""

import os
import sys

from setuptools import setup, find_packages

needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []

DOCLINES = (__doc__ or '').split("\n")

with open(os.path.join(os.path.dirname(__file__),
                       'VERSION'), 'r') as version_file:
    version = version_file.read().strip()

setup(name='workingtime',
      version=version,
      description=DOCLINES[1],
      long_description="\n".join(DOCLINES[3:]),
      url='https://workingtime.readthedocs.io',
      maintainer='Pablo Manso',
      maintainer_email='92manso@gmail.com',
      include_package_data=True,
      platforms=['any'],
      license='MIT',
      packages=find_packages(),
      python_requires='>=3.6, <4',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7'
      ],
      install_requires=['numpy>=1.16', 'pandas'],
      setup_requires=pytest_runner,
      tests_require=['pytest'],
      test_suite='tests',
      zip_safe=False)



import unittest

import numpy as np
from datetime import timedelta, datetime
from workingtime.workingtime import _dates_between_dates
from workingtime import WorkingTime, Time


class InitialTest(unittest.TestCase):

    def test_dates_between_dates(self):
        now = datetime.now()
        day = timedelta(1)

        np.testing.assert_array_equal(
            _dates_between_dates(now, now), [])
        np.testing.assert_array_equal(
            _dates_between_dates(now, now + day), [])
        np.testing.assert_array_equal(
            _dates_between_dates(now, now + 2 * day),
            [now + day])

    def test_another_test(self):
        wt = WorkingTime()
        np.testing.assert_equal(
            wt.working_time(datetime(2020, 10, 3, 1),
                           datetime(2020, 10, 4, 23),
                           (Time(22, 0), Time(0, 30))).total_seconds(),
            (2.5 + 1) * 3600)


if __name__ == '__main__':
    print()
    unittest.main()
